# Plugin Repositories Specification

**Owners:**
Pepper Lebeck-Jobe [@eljobe](https://github.com/eljobe)
Stefan Oehme [@oehme](https://github.com/oehme)   
**Updated:** 2016-05-23   
![Approved](https://img.shields.io/badge/design-approved-green.svg)

<!-- START doctoc generated TOC please keep comment here to allow auto update -->
<!-- DON'T EDIT THIS SECTION, INSTEAD RE-RUN doctoc TO UPDATE -->


- [Objective](#objective)
  - [Goals / Use Cases](#goals--use-cases)
  - [Non-Goals / Out of Scope Use Cases](#non-goals--out-of-scope-use-cases)
- [Background](#background)
- [Solution](#solution)
  - [A pluginRepositories {} block in settings.gradle or an init script](#a-pluginrepositories--block-in-settingsgradle-or-an-init-script)
    - [Implementation Plan](#implementation-plan)
  - [Add an opt-in mechanism for the Gradle Plugin Portal](#add-an-opt-in-mechanism-for-the-gradle-plugin-portal)
- [Appendix A: Historical Discussion](#appendix-a-historical-discussion)
  - [Is buildscript {} deprecated?](#is-buildscript--deprecated)
  - [Alternative Designs Considered](#alternative-designs-considered)
  - [Rejected](#rejected)

<!-- END doctoc generated TOC please keep comment here to allow auto update -->

## Objective

Milestone 1.2 of the work related to the
[stable-plugins-dsl project](https://github.com/gradle/stable-plugins-dsl)
is defining a way for builds to specify which maven/ivy repositories
should be searched for plugins.

### Goals / Use Cases

* As a developer on a project, I want to be able to use plugins maintained by my company’s infrastructure team using the `plugins {}` DSL, without knowing the names of the artifact repositories which provide these plugins
* As a member of the infrastructure team, I want to be able to specify which repositories to use for plugin resolution for a whole Gradle build, so that internal plugins can be consumed using the `plugins {}` DSL
* As a member of the infrastructure team at a company in a highly-regulated industry, I need to be able to exclude the Gradle Plugins Portal as a source of plugins, so that I can completely control and audit the dependencies used by the development teams I support.

### Non-Goals / Out of Scope Use Cases

* As a member of the infrastructure team, I want to be able to apply plugins to the `Gradle` and `Settings` instances, not just to `Project`s
* As a member of the infrastructure team, I want to apply plugins to projects from an init script without adding the plugin repository to the list of the plugin repositories for that build. I.e. I want to resolve them in isolation.
* As a member of the infrastructure team, I want to be able to centrally maintain the list of plugin repositories in an init script, which is referenced from all projects.
    * This use case is going to be tackled in the future [Build Platform](https://github.com/gradle/gradle/tree/master/design-docs/features/build-platform) work. It is mentioned here to ensure that this design is "forward compatible" with such work.
* As a developer on a project, I want to be able to override the plugin repositories used to resolve plugins on a project and subproject level.
    * We consider this use case overly complex and intend not to support it. The plugin repositories are a build-scoped declaration which does not need to be altered from project to project on a multi-project build.

## Background

The old way to specify where to find
[Gradle plugins](https://docs.gradle.org/current/userguide/plugins.html)
was to use a buildscript block to identify the repositories which
hosted the artifacts which contained your plugins, and the specific
dependencies you needed from those repositories, and then applying
your plugin by its id or class name. For example:

```gradle
buildscript {
    repositories {
        jcenter()
    }
    dependencies {
        classpath "com.jfrog.bintray.gradle:gradle-bintray-plugin:0.4.1"
    }
}
```

apply plugin: "com.jfrog.bintray"

Recently, we’ve introduced a new feature of our Domain Specific
Language (DSL) called `plugin`s which allows users to simply use the
`id` and `version` number of their desired plugin, and have the
corresponding artifact added to the buildscript classpath using the
[Gradle Plugin Portal](http://plugins.gradle.org).  For example:

```gradle
plugins {
    id "com.jfrog.bintray" version "0.4.1"
}
```

This new DSL feature has worked very well for publicly available
Gradle plugins, but it doesn’t allow an alternative location for
organizations which don’t want to share their plugins with the world
on the Gradle Plugin Portal.

This design is part of a larger effort to stabilize the new plugins
DSL. The overall design calls for two high-level technical
enhancements to Gradle:

* **M1** Resolve plugins from a maven/ivy repository based on the plugin id and version
    * **[M1.1](M1.1.md)** Be able to map from plugin id and version to a maven/ivy artifact
    * **[M1.2](M1.2.md)** Specify which maven/ivy repositories to search for plugins
    * **[M1.3](M1.3.md)** Publish plugins to a maven/ivy repository complete with metadata needed to map from the plugin id and version to a maven/ivy artifact and its dependencies
* **M2** Specify plugins to be used on multiple projects in a multi-project build in a single code block

This design is concerned with milestone M1.2 of this list of milestones.

## Solution

This design calls for two high-level technical changes to the syntax and semantics of the DSL. Each is described in detail in this section.

1. Introduce a `pluginRepositories {}` block in `settings.gradle`/`init.gradle`
1. Add an opt-in mechanism for the [Gradle Plugin Portal](https://plugins.gradle.org)

### A pluginRepositories {} block in settings.gradle or an init script

This design introduces a `pluginRepositories` block in the DSL. It can
only be specified in a `settings.gradle` file or an init script. For example:

```gradle
pluginRepositories {
    maven {
        url "http://repo.mycompany.com/maven2"
    }
    gradlePluginsPortal()
    maven {
        url "http://repo.backstop.com/maven2"
    }
}
```

The `pluginRepositories` section is only valid in the scope of an entire
build. So, it cannot be supplied at the project level.

#### Implementation Plan

Each repository in the pluginRepositories block results in adding a
`PluginRepository` which adds a `PluginResolver` instance to the list
of `PlugResolver`s used to resolve `PluginRequest` objects.  In the
case of maven or ivy repositories, the `PluginResolver` is an instance
of `ArtifactRepositoryPluginResolver`. In the case of the
`gradlePluginsPortal()` it is an instance of
`PluginResolutionServiceResolver`.

This design gives us the best control over reporting the places we
looked during an attempt to resolve a plugin, and gives us a simple
mechanism for ordering the plugin sources we should search.

### Add an opt-in mechanism for the Gradle Plugin Portal

If there is no `pluginRepositories` block specified for a build, we will
default to a `pluginRepositories` block which only includes the [Gradle
Plugins Repository](https://plugins.gradle.org). However, specifying a `pluginRepositories` block
will essentially overwrite this default with the list of repositories
specified. This allows us to support the third
[use case listed above](#goals--use-cases) in which an
organization wants to avoid resolving artifacts from untrusted sources
like the Gradle Plugins Portal.

If `pluginRepositories` blocks are specified in multiple places, they
will be accumulated in the order the specifications are evaluated by
Gradle. The order of evaluation is:

1. From init scripts
1. From settings.gradle

Plugin repositories specified later in the evaluation order will
simply be appended to the ones specified earlier in the evaluation
order.

Declaring plugin repositories in `build.gradle` files will not be
supported. Since the list of plugin repositories is build-global, it
would only make sense to allow it in the root project. But allowing it
there might lead users to believe that they can also use it in
subprojects. We want to avoid this confusion. Also, we believe that
the intersection between "projects that don’t have a `settings.gradle`"
and "projects that need an enterprise plugin repository" is
essentially zero.

## Appendix A: Historical Discussion

This is just back-matter related to discussions we had comparing various alternate designs.

### Is buildscript {} deprecated?

There have been some questions related to this design and the
suggestion below that we want to move away from the `buildscript {}`
block in the future. It is true that we want to replace all of the
functionality currently available in the `buildscript {}` block with
alternative implementations. But, this design is not sufficient to
deprecate the `buildscript {}` block. It merely provides a better way of
describing the repositories from which plugins should be
resolved. Other projects in the future will provide better ways to
accomplish the other use cases currently served by the `buildscript {}`
block. The only reason our general movement away from the `buildscript
{}` block is mentioned in this document is that it impacts the fitness
of certain solutions in the context of our future strategic goals.

### Alternative Designs Considered

**Add a pluginRepositories {} block in build.gradle/settings.gradle**

This is decoupled from the `plugins` block in the same way the
`repositories` and `dependencies` sections of a project are decoupled.

**Pros:**

* It is more explicit that we are only configuring the plugin repositories
* It could be set from an init script, which works nicely with the build platform idea mentioned earlier
* It may be a little easier to understand that this block is not supported on subrpojects.

**Cons:**

* It doesn’t make sense if there isn’t also a plugins block in the build script. (Why did you specify a pluginRepository if you didn’t want any plugins?)
* Plugin information is no longer accessible to the viewer in one place
* Having a global list of plugin repositories has a negative implication on performance. Each repository has to be searched, in order to see it each plugin is defined in that repository.
* Having a global list plugin repositories makes it possible that the unintended plugin could be resolved because a plugin with the same id and version exists in a different repository with different contents, and that repository is earlier in the list of plugin repositories.

### Rejected

**Add a repositories {} block to the plugins {} block**

Similar to how you specify your build dependencies right next to the
repositories from which you are going to resolve those dependencies,
we could just specify the repositories which can hold plugins. The
repositories block would be allowed in `settings.gradle`, and prohibited
in subprojects. All declarations of plugin repositories would
contribute to one global list of plugin repositories, so they don't
need to be repeated in subprojects.

**Pros:**

* Colocation with the plugins implies which repositories you are specifying.
* Idiomatically similar to existing mechanisms from initscript and buildscript.

**Cons:**

* This solution implies to the user that he can have different repositories for different projects, which is not what we currently have in mind (repositories being build-scoped)
* This solution makes it difficult to understand where you can provide which declarations. It doesn't make sense to apply a project plugin at the level of the settings.gradle file, but that might be a good place to specify the plugin repositories. So, it really conflates two pieces of build configuration which would make more sense separated.
* Because there are different types of plugins which make sense at different scopes of the build model, we want to be very deliberate on how we introduce the ability to declare plugins with init scripts and settings.gradle files. Going with this solution would complect the two issues and we needn't.

**Just use buildscript/initscript { repositories {} }**

This list of repositories is already available in our DSL and it
matches well with our intended goal. Users could just add their
plugins repository among the other repositories specified in that
block.

**Pros:**

* The mechanism already exists.

**Cons:**

* The terms `buildscript` and `initscript` don’t make a connection to the plugins-dsl
* Adding more repositories affects how dependency resolution works and hence the number of repositories that are interrogated when resolving. When the build script needs a plugin it will check all those repos, including ones that are not housing plugins.
* We want to get rid of the buildscript block in the long run

**Support named repositories and a way to reference**

```gradle
repositories {
  ourCorporateReleaseRepo {
    maven { }
  }
  ourExperimentalRepo {
    maven { }
  }
}

def pluginRepos = [repositories.ourCorporateReleaseRepo, repositories.ourExperimentalRepo]

plugins {
    repositories pluginRepos
    id "com.jfrog.bintray" version "0.4.1" 
}
```

Or maybe:

```gradle
plugins {
    id "com.jfrog.bintray" version "0.4.1" from jcenter()
    id "com.foo"  version "1.2.3" from {
        jcenter()
        mavenCentral()
    }
}
```

**Pros:**

* Allows sharing repository definitions.
* Allows optimizing the number of repositories which have to be checked when searching for a plugin

**Cons:**

* The plugins block is evaluated before the build script, so this wouldn’t work.

**Have a completely different file that is evaluated before initscript/settings.grade/build.grade**

**Pros:**

* ???

**Cons:**

* It isn’t clear why this should be separate from settings.gradle
* More files that users need to read to understand their build

**Allow a pluginsRepository property in wrapper.properties**

This would allow the project owners to specify a list of plugin repositories to check and it would work as long as everyone is using the wrapper to build the project.

**Pros:**

* This is what a primary stakeholder proposed because they already have an existing `wrapper.properties` update mechanism that they could leverage to also update the plugin repositories

**Cons:**

* This isn’t really about configuring the instance of Gradle that is being used, and so, seems out of scope.
* Build logic cannot influence the location, (e.g. when a different repository url or credentials should be used when the build is run from different locations. We've seen this with real customers.)
* When there are multiple repositories that should be considered, eg when a repo of stable plugins must be used by the CI builds, but dev builds can use an additional repo of unstable or non-vetted plugins. We've seen this with a real customer.
* When the public plugin portal must **not** be used. We've seen this for example at a large financial company.
* There are other ways customers can accomplish centralized maintenance of which repositories should be used.

**Add a pluginRepositories {} block to the buildScript {} block**

**Pros:**

* Under the hood it could use some, if not all of, the existing plumbing provided by the `repositories {}` block
* Seems to match the problem "repositories from which a build should attempt to download plugins" 

**Cons:**

* We want to get rid of the `buildscript{}` block in the future
